@@ -21,6 +21,9 @@
 Data manipulation
 =================
 """
+import types
+
+
 def merged(spec, data, value_processor=lambda x:x):
     """ Returns a dictionary based on `spec` + `data`.
 
@@ -44,16 +47,20 @@ def merged(spec, data, value_processor=lambda x:x):
             if key in data:
                 if data[key] is None:
                     value = spec[key]
+                    print key, 'in data was none -> default', value
                 else:
                     value = data[key]
+                    print 'got', key, 'from data', value
             else:
                 value = spec[key]
+                print key, 'was not in data -> default', value
 
-            # special handling of dict and list instances
+            # special handling of dict instances, list instances and callables
             if (spec[key] == dict or isinstance(spec[key], dict)) and isinstance(value, dict):
                 # nested dictionary
                 value = merged(spec.get(key, {}), value)
-            if (spec[key] == list or isinstance(spec[key], list)) and isinstance(value, list):
+                print 'merged'
+            elif (spec[key] == list or isinstance(spec[key], list)) and isinstance(value, list):
                 # nested list
                 item_spec = spec[key][0] if spec[key] else None
                 if isinstance(item_spec, type):
@@ -69,6 +76,20 @@ def merged(spec, data, value_processor=lambda x:x):
                 else:
                     # probably default list item like [1]
                     pass
+            elif isinstance(spec[key], (types.FunctionType, types.BuiltinFunctionType)):
+                # default value is obtained from a function with no arguments;
+                # (It is expected that the callable does not have side effects.)
+                if hasattr(value, '__call__'):
+                    # FIXME this is unreliable: the value may be already
+                    # a result of calling the function from spec which, in
+                    # turn, can be callable. Instead of checking for __call__
+                    # we should check if the value was obtained from data or
+                    # from the spec. This is problematic at the moment because
+                    # nested structures are simply assigned to `value` if
+                    # `value` is None or is not in the data, and *then* the
+                    # structure is recursively merged (at which point the
+                    # information on the source of given chunk of data is lost)
+                    value = value()
             else:
                 # some value from spec that can be checked for type
                 pass
